#!/usr/bin/perl
use strict;

use FindBin qw($Bin);
use File::Basename;

my $window_size = 500;
my $shift_length = 50;

my $pos_fa    = $ARGV[0]; #testdata/pos.fa
my $neg_fa    = $ARGV[1]; #testdata/neg.fa
my $target_fa = $ARGV[2]; #testdata/target.fa
my $outdir    = $ARGV[3]; #outdir

if(!defined($pos_fa) || !defined($neg_fa) || !defined($target_fa) || !defined($outdir)){
	print STDERR "USAGE:\n./score.pl <pos.fa> <neg.fa> <target.fa> <outdir>\nPlease check README file for more information.\n";
	exit;
}

mkdir($outdir);

my $pos_name = basename($pos_fa);
my $neg_name = basename($neg_fa);
my $target_name = basename($target_fa);
print STDERR "Building positive model\n";
system("$Bin/src/bin/Build-IMM -r -k 6 < $pos_fa > $outdir/model.pos");

print STDERR "Building negative model\n";
system("$Bin/src/bin/Build-IMM -r -k 6 < $neg_fa > $outdir/model.neg");

print STDERR "Processing target file\n";
system("$Bin/src/preproc_target.pl -w_dist 0 $target_fa $outdir $window_size $shift_length");

print STDERR "Scoring the target\n";
system("$Bin/src/bin/Score -n $outdir/model.neg $outdir/model.pos $outdir/tmp_target > $outdir/score.IMM");

print STDERR "Score file is located at: outdir/score.IMM\n";
