#include "Score.h"

map<string, double> immhash;
map<string, double> neg_immhash;

int Model_Depth =6;
int neg_Model_Depth =6;
int Model_col_idx = 3; //the column index of the model file. 3=IMM, 2=fixed model
bool Score_Reverse =false;
bool use_neg_Model=false;
char* neg_Model_file;


int main(int argc, char * argv []){

	Process_Options (argc, argv);
	immhash = Read_Model(argv[argc-2], Model_Depth);
	if(use_neg_Model){
		neg_immhash=Read_Model(neg_Model_file, neg_Model_Depth);
	}
	Score_Sequences((char*)argv[argc-1]);
	return 0;
}

map<string, double> Read_Model(char* filename, int& depth){

        FILE* fpmodel = fopen (filename, "r");
        if(fpmodel == NULL){
                cout<<"ERROR: Could not open model-file "<<filename<<endl;
                exit(1);
        }
	map<string, double> rethash;
	
	char* p;
	char line [MAX_LINE];

	while(fgets(line, MAX_LINE, fpmodel)!=NULL){
		int idx=0;
		string str;
		double imm=0;	
		p = strtok (line, " \t\n");
		while (p != NULL){
			if (idx==0) str = p;
			if (idx==Model_col_idx) imm = atof(p);
			p = strtok (NULL, " \t\n");
			idx++;
		}

		rethash.insert(pair<string,double>(str,imm));
		depth = str.length();
	}
	fclose(fpmodel);
	return rethash;
}

void Score_Sequences(char* filename){
	Sequence seq (filename);
	get_Sequence_Score(seq._seq, seq._name);
	while(seq.Next_Seq()){
		get_Sequence_Score(seq._seq, seq._name);
	}
}

void get_Sequence_Score(string seq, string seqname){

	double retval = Score_Sequence(seq,seqname);
	if(Score_Reverse){
		Sequence tmp(seq,"tmp");
		tmp.Reverse_Complement();
		retval+= Score_Sequence(tmp._seq,seqname);
	}

	cout<<seqname<<" "<<retval<<endl;
} 

double Score_Sequence(string seq, string seqname){

	double psum =0;	
	for(int i=0; i <= seq.length()-Model_Depth; i++){
		string str;
		str.assign(seq, i, Model_Depth);
		map<string,double>::iterator it;
		it=immhash.find(str);
		
		double tsum =0; 
		if(it!=immhash.end())
			tsum=it->second;
		if(tsum>0)
			psum+=log(tsum);
		else
			psum+=-2; //log(0.01)
	}

	if(!use_neg_Model) return psum;

	double nsum =0;
	for(int i=0; i <= seq.length()-neg_Model_Depth; i++){
		string str;
		str.assign(seq, i, neg_Model_Depth);
		map<string,double>::iterator it;
		it=neg_immhash.find(str);

		double tsum=0;
		if(it!=neg_immhash.end())
			tsum=it->second;
		if(tsum>0)
			nsum+=log(tsum);
		else
			nsum+=-2; //log(0.01)
	}
	return (psum-nsum);
}

///
double Score_SequenceEX(string seq, string seqname){

	double sum =0;	

	for(int i=0; i <= seq.length()-Model_Depth; i++){
		string str;
		str.assign(seq, i, Model_Depth);
		map<string,double>::iterator it;
		it=immhash.find(str);
		
		double psum =0; 
		if(it!=immhash.end())
			psum=it->second;

		if(neg_immhash.size()){ // negative model is provided
			double nsum=0;
			it=neg_immhash.find(str);
			if(it!=neg_immhash.end())
				nsum=it->second;
			if(psum && nsum){
				sum+= log(psum/nsum);
			}
				
		}else{
			if(psum>0)
				sum+=log(psum);
		}


	}

	return sum;
}

void  Process_Options (int argc, char * argv []){
//  Get options and parameters from command line.

   int  errflg = FALSE;
   int  ch;
   char  * p, * q;
   FILE* fp;

   optarg = NULL;
   while(! errflg && ((ch = getopt (argc, argv, "n:fr")) != EOF)){
     switch(ch){
        case 'n':    // read negative model file
	  use_neg_Model =true;
	  neg_Model_file = optarg;
          break;
	case 'f':
	  Model_col_idx=2;
	  break;
	case 'r':
	  Score_Reverse =true;
	  break;
        case '?':
          fprintf (stderr, "Unrecognized option -%c\n", optopt);
        default :
          errflg = TRUE;
     }
   }
   if(errflg || argc < 3 || argc< optind + 2){
        Usage ();
        exit (EXIT_FAILURE);
   }
   return;
}


static void  Usage (){
//  Print the description of options and command line for this program.

   fprintf (stderr,
       "USAGE:  Score [options] <model-file> <sequence-file>\n"
       "Reads sequences from sequence-file and scores every Fasta sequence\n"
       "using the given model. The scores are printed in standard output.\n\n"
       "Options:\n"
       " -n <filename>\n"
       "    Reads the model on negative sequences from <filename> and outputs the LL score\n"
       " -f \n"
       "    for using fixed kmer model instead of IMM use this option\n"
       " -r \n"
       "    score sequence and its reverse complement\n");
   return;
}

